package World;

import Units.Player;

public class Map {
    
    private Location[][] board;
    
    public Map(int width, int height) {
        board = new Location[height][width];
    }
    
    //create a map with a default size
    public Map() {
        //call the other constructor to do the work
        this(10, 10);
    }
    
    public Location getLocationAtPosition(int x, int y) {
        if (x < 0 || y < 0 || y >= board.length || x >= board[y].length) {
            return null;
        }else{        
            return board[y][x];
        }
    }
    
    public void setLocation(int x, int y, Location location) {
        if (x >= 0 && y >= 0 && y < board.length && x < board[y].length) {
            board[y][x] = location;
        }
    }
    
    public void printMap(Player player) {
        String boardText = "";
        
        for (int y = 0; y < board.length; y++) {
            for (int x = 0; x < board[y].length; x++) {
                if (player.getX() == x && player.getY() == y) {
                    //print out the player
                    boardText += 'P';
                }else{                
                    Location location = board[y][x];
                    boardText += getLocationChar(location);
                }
            }
            boardText += "\n";
        }
        
        System.out.print(boardText);
    }
    
    private char getLocationChar(Location location) {
        if (location == null) {
            //print out a missing location
            return '.';
        }else{
            //print out the map symbol
            return location.getMapSymbol();
        }        
    }
    
    public void createOpenMap() {
        for (int y = 1; y < board.length - 1; y++) {
            for (int x = 1; x < board[y].length - 1; x++) {
                board[y][x] = new Location(' ').setSolid(false);
            }
        }
        
    }
}