package dragons;

import java.util.ArrayList;
import java.util.EnumSet;

import java.awt.Color;
import java.awt.Point;
import java.awt.Graphics2D;
import java.util.ArrayList;

import particles.FireParticle;
import gui.FireIcon;
import gui.HeartIcon;

import main.Game;
import main.IGraphicalObject;
import main.RENDERPASS;

public abstract class Dragon implements IGraphicalObject{
    //the current position and speed of the dragon
    private float x;
    private float y;
    private float speedX;
    private float speedY;
   
    
    //the current rotation of the dragon
    private float rotationBody;
    
    //the rotations at the different joints in the legs of the dragon
    private float rotationBackLegUnder;
    private float rotationBackLegUpper;
    private float rotationFrontLegUnder;
    private float rotationFrontLegUpper;
    
    //keeps track of the animation progress
    private float moveTick;
    
    //handles the opening of the mouth by controlling the angles of the jaws
    private float rotationJawUnder;
    private float rotationJawUpper;
    
    //controls the movement in the tail of the dragon
    private float tailMovement;
    
    //controls the health of the dragon
    private int maxHealth;
    private float health;
    
    //controls the fire bar of the dragon
    private float fireMeter;
    private int fireMeterMax;
    
    //keeps track of all fire particles
    private ArrayList<FireParticle> fires;
    
    //keeps track of the fire animation
    private int fireMode;      
    
    //the wings' height offset depending on their current flapping state
    private int flapOffset;
    
    //controls the flapping
    private float flapAngle;
    
    
    //remember the last state of the wings, this is to be able to push the dragon upwards properly
    private boolean wingWasDown;

    //used to control the dragon
    protected boolean left;
    protected boolean right;
    protected boolean fire;
    protected boolean flying;    

    //creates a dragon at the specific location with the given health and fire meter
    public Dragon(int x, int y, int health, int fire) {                
        
        //nollstller drakens rrelsemnster 
        resetMovement();
        
        //frbereder eldanimationen
        fires = new ArrayList<FireParticle>();
        fireMode = -1;   
        
        //stnger munnen
        rotationJawUnder = 0;
        rotationJawUpper = 0;  
        
        //frbereder vingen och flygningen
        flapAngle = (float)(Math.PI * 2);
        wingWasDown = true;
        flying = false;

        
        this.x = x;
        this.y = y;
        this.health = this.maxHealth = health;
        this.fireMeter = this.fireMeterMax = fire;
        
        
        Game.instance.objects.add(this);
    }
    
    
    public ArrayList<FireParticle> getFireList() {
        return (ArrayList<FireParticle>)fires;
    }
    
    public boolean isDead() {
        return health <= 0;
    }
    
    public float getSpeedX() {
        return speedX;
    }
    
    public float getSpeedY() {
        return speedY;
    }
    
    
    //metod som nollstller drakens rrelsemnster
    private void resetMovement() {
        moveTick = -1;
        
        //ge draken rtt vinkel             
        rotationBody = -0.15F;
        
        //stt rtt startvinklar p benens leder
        rotationBackLegUnder = (float)(1.2F);
        rotationBackLegUpper = (float)(Math.PI*9/4.0);
        rotationFrontLegUnder = (float)(0.5F - 0.03*30);
        rotationFrontLegUpper = (float)(Math.PI*7/4.0);      
        
        //nollstll svansens rrelse
        tailMovement = 0;
    }
    

    
    public int getMaxHealth() {
        return maxHealth;
    }
    
    public float getHealth() {
        return health;
    }
    
    
    
    public void damageMe(float val) {
        val = Math.max(val,0);
        health -= val;
        if (health < 0) {
            health = 0;
            
            onDeath();
        }
    }
    
    protected void onDeath() {
        
    }
    
    public void healMe(float val) {
        val = Math.max(val,0);
        health += val;
        
        if (health > maxHealth) {
            health = maxHealth;
        }
    }
    
    
    public int getFireMeterMax() {
        return fireMeterMax;
    }
    
    public float getFireMeter() {
        return fireMeter;
    }
    
    
    
    public void useFire(float val) {
        val = Math.max(val,0);
        fireMeter -= val;
        if (fireMeter < 0) {
            fireMeter = 0;
        }
    }
    
    public void restoreFire(float val) {
        val = Math.max(val,0);
        fireMeter += val;
        
        if (fireMeter > fireMeterMax) {
            fireMeter = fireMeterMax;
        }
    }     
    
    public abstract Color PrimaryColor();
    public abstract Color SecondaryColor();
    public abstract Color BorderColor();
    
    private float wingspeed = 0.1F;
    private final float gravitation = 0.1F;
    
    //updates the dragon
    @Override
    public void update() {
        //updates the speed
        speedY += gravitation;   
        
        //update the position
        y += speedY;
        
        //crash in the ground
        if (y > 470) {
            //take damage if the dragon crashes too fast
            if (speedY > 4) {
                damageMe(0.2F + (speedY-4)/5);
            }            
            
            speedY = 0;
            y = 470;
        }
        
        //if both the right and left buttons are being held of if none of them are held down, continue as normal
        if (right == left) {
            //apply airresistance
            if (speedX < 0) {
                speedX += 0.1;
                if (speedX > 0) {
                    speedX = 0;
                }
            }else if (speedX > 0) {
                speedX -= 0.1;
                if (speedX < 0) {
                    speedX = 0;
                }
            }
            
            
        }else if(right) {
            //move to the right
            speedX = 4;
                        
        }else if(left) {
            //move to the left
            speedX = -4;
        }
        
        //update the position
        x += speedX;
           
        //how fast the legs should be moving
        float  mult; 
        if (speedX == 0 || inAir()) {
            mult = 0;
        }else{
            mult = 2 * Math.max(1F,Math.abs(speedX)); 
        }
        
        int stopAt = 8;
        
        //the dragon will only move the legs when it's running or falling, furtheremore the legs should reach a specific state before they stop
        if (mult != 0 || moveTick != stopAt) {
            if (mult == 0) {
                mult = 2;
            }
            
            if( moveTick < 30) {
                //part one of the animation
                rotationBody += 0.005*mult;
                tailMovement+=0.03F*mult;
                rotationFrontLegUpper += 0.03F*mult;
                rotationFrontLegUnder += 0.03F*mult;
                rotationBackLegUpper += 0.03F*mult;
                rotationBackLegUnder -= 0.03F*mult;
                
                //part two of the animation
            }else if(moveTick < 60) {
                rotationBody -= 0.005*mult;
                tailMovement-=0.03F*mult;
                rotationFrontLegUpper -= 0.03F*mult;
                rotationFrontLegUnder += 0.03F*mult;
                rotationBackLegUpper -= 0.03F*mult;
                rotationBackLegUnder -= 0.03F*mult;  
                
                //part three of the animation
            }else if(moveTick < 90) {   
                rotationBody -= 0.0025*mult;
                tailMovement-=0.03F*mult;
                rotationFrontLegUnder -= 0.03F*mult;  
                rotationBackLegUnder += 0.03F*mult;    
                
                //part four of the animation
            }else if(moveTick < 120) {
                rotationBody += 0.0025*mult;
                tailMovement+=0.03F*mult;
                rotationFrontLegUnder -= 0.03F*mult; 
                rotationBackLegUnder += 0.03F*mult;  
                
                //restart
            }else{
                resetMovement();   
            }               
            
            //the show must go on
            moveTick+=mult;
            if (moveTick > stopAt && moveTick-mult < stopAt) {
                moveTick = stopAt;
            }
            
        }
        
        //update teh fire particle
        for(int i = fires.size()-1; i >= 0; i--) {
            if (!fires.get(i).update()) {
                fires.remove(i);
            }
        }        
        if (fireMode == -1 && fire) {
            fireMode = 0;
        }
        
        boolean isSpittingFire = false;
        if(fireMode != -1) {
            //open the mouth
            if(fireMode < 30) {
                rotationJawUnder -= 0.01F;
                rotationJawUpper += 0.01F;  
                
                //spit fire
            }else if(fireMode < 50) {
                for(int i = 0; i < 20; i++) {
                    Point p;
                    int r = 47;
                    float a = rotationBody; 
                    boolean invert;
                    if (lookingLeft()) {
                        p = new Point((int)x - (int)(Math.cos(a)*r), (int)y - 42 -(int)(Math.sin(a) * r));
                        invert = false;
                    }else{
                        p = new Point((int)x + (int)(Math.cos(a)*r), (int)y - 42 - (int)(Math.sin(a) * r));
                        invert = true;
                    }
                    fires.add(new FireParticle(this,p.x,p.y,a,invert,(int)speedX,(int)speedY));         
                }
                useFire(0.02F);
                isSpittingFire = true;
                if (fireMeter == 0) {
                    
                    fireMode = 50 - 1;
                }else if (fireMode == 50 - 1 && fire) {
                    fireMode--;
                }
                
                //close the mouth
            }else if(fireMode < 80) {
                rotationJawUnder += 0.01F;
                rotationJawUpper -= 0.01F;  
                
                //stop the animation
            }else{
                fireMode = -2;
            }
            
            //the show must go on
            fireMode++;
        }
        if (!isSpittingFire) {
            restoreFire(0.01F);
        }
        
        float oldFlapAngle = flapAngle;
        
        //update the wings
        
        
        //if the dragon is flying the wings should continue to flap
        if (flying) {
            flapAngle = (float)((flapAngle + wingspeed));
            flapAngle = (float)(flapAngle % (Math.PI));
            
            //otherwise it should stop in the idle state(only stops every second time though)
        }else if(flapAngle != 0) {
            flapAngle = (float)((flapAngle + wingspeed));
            
            if(flapAngle > Math.PI) {
                flapAngle = 0;
            }
        }
        
        //calculate where the wings should be
        flapOffset = (int)Math.abs((Math.cos(flapAngle) * 50));
        
        
        //calculate if the wings are in the down position
        boolean wingIsDown = flapOffset >= 45;
        
        //if the wings are down but weren't the last time, allow the wings to push air downwards
        if(wingIsDown && !wingWasDown) {
            speedY -= 4;
        }
        
        //saves the state for later
        wingWasDown = wingIsDown;
    }
    
    public boolean getIsFlapping() {
        return flapAngle != 0;
    }
    
    public int flapTime() {
        return (int)Math.ceil(Math.acos(5.0F / 50.0F) / wingspeed);
    }
    
    public float getYAtFlap() {
        int time =  flapTime();
        return y + speedY * time + (float)Math.pow(time,2) * gravitation / 2;
    }
    
    public float getSpeedYAtFlap() {
        int time =  flapTime();
        return speedY + gravitation * time;
    }    
    
    //if the dragon is moving, update the direction its facing depending on the speed, if the dragon is not moving just use the last direction the dragon was facing
    private boolean lookingleft = false;
    private boolean lookingLeft() {
        if (speedX != 0) {
            lookingleft = speedX < 0;
        }
        return lookingleft;
    }
    
    //defines if we're in the air or not
    private boolean inAir() {
        return y < 470;
    }
    
    public float getX() {
        return x;
    }
    public float getY() {
        return y;
    }
    
    //draw a rectangle with the given background color and border color
    private void paintPart(Graphics2D g, Color background, Color border, int x, int y, int w, int h) {  
        g.setColor(background);    
        g.fillRect(x, y, w, h);
        g.setColor(border);               
        g.drawRect(x, y, w, h);
    }
    
    
    //draw the dragon
    @Override
    public void draw(Graphics2D g, RENDERPASS pass) { 
        //round the position to integer since we draw pixels anyways
        int x = (int)this.x;
        int y = (int)this.y;
        
        //draw all particles that are behind the dragon
        for(FireParticle f: fires) {
            f.Draw(g, false);
        }       
        
        //if the dragon is facing right, invert it
        if (!lookingLeft()) {
            g.scale(-1, 1);
            g.translate(-x * 2, 0);
        }
        
        //apply the dragon's rotation
        g.rotate(rotationBody, x, y - 40 + 25/2);  
        
        //get the colors to draw the dragon
        Color shellPrimary = PrimaryColor();
        Color shellSecondary = SecondaryColor();
        Color shellBorder = BorderColor();
        
        
        //draw the neck
        paintPart(g, shellPrimary,shellBorder, x - 26, y - 45, 15, 11);
        
        //draw teh body                
        paintPart(g, shellPrimary, shellBorder, x - 20, y - 40, 40, 25);
        
        //draw the tail                
        //calculate the position of the next tail segment
        int oX = 0;
        float oY = 0;
        
        //loop through the tail segments and draw them
        for(int i = 0; i < 18; i++) {
            //calculate teh size of this segment
            int s = 11 - i/2;                      
            
            //draw the segment
            paintPart(g, shellPrimary, shellBorder, x + 20 + oX, (int)(y - 40 + oY), s, s);
            
            //calcalute the position of the next segment
            oX +=s*0.5;
            oY +=s*0.5 - i/(5.0 + tailMovement) + tailMovement/2.0;
            
        }
        
        //draw the head
        paintPart(g, shellPrimary, shellBorder, x - 40, y - 50, 15, 15);
        
        //draw the upper jaw at the given rotation
        g.rotate(rotationJawUpper, x - 40 - 5, y - 50 + 3);  
        paintPart(g, shellPrimary, shellBorder, x - 50, y - 52, 5, 2);
        paintPart(g, shellPrimary, shellBorder, x - 52, y - 50, 12, 7);
        g.rotate(-rotationJawUpper, x - 40 - 5, y - 50 + 3);  
        
        //draw the lower jaw at the given rotation
        g.rotate(rotationJawUnder, x - 40 - 5, y - 50 + 10);  
        paintPart(g, shellPrimary, shellBorder, x - 52, y - 42, 12, 7);
        g.rotate(-rotationJawUnder, x - 40 - 5, y - 50 + 10); 
        
        //draw the ear
        paintPart(g, shellPrimary, shellBorder, x - 32, y - 55, 7, 5);
        
        //draw the eye
        paintPart(g, shellBorder, shellBorder, x - 38, y - 48, 3, 3);
        
        
        //draw the upper part of the front leg
        g.rotate(rotationFrontLegUpper, x - 15, y - 25);  
        paintPart(g, shellPrimary, shellBorder, x - 15 - 13, y - 25 - 5, 18, 10);
        
        //draw the lower part of the front leg
        g.rotate(rotationFrontLegUnder, x - 30 + 5, y - 30 + 5);                 
        paintPart(g, shellPrimary, shellBorder, x - 30, y - 30, 10, 10);
        paintPart(g, shellPrimary, shellBorder, x - 30, y - 20, 8, 3);
        paintPart(g, shellPrimary, shellBorder, x - 35, y - 17, 10, 5);
        g.rotate(-rotationFrontLegUnder, x - 30 + 5, y - 30 + 5);   
        
        //reset the rotation for the front leg
        g.rotate(-rotationFrontLegUpper, x - 15, y - 25); 
        
        
        //draw the upper part of the hind leg
        g.rotate(rotationBackLegUpper, x + 5, y - 30);  
        paintPart(g, shellPrimary, shellBorder, x, y - 30 - 8, 25, 16);
        
        
        //draw the lower part of the hind leg
        g.rotate(rotationBackLegUnder, x + 30 + 5 - 15, y - 36 + 5);  
        paintPart(g, shellPrimary, shellBorder, x + 30 - 15, y - 38, 20, 12);
        paintPart(g, shellPrimary, shellBorder, x + 30 - 15 + 20, y - 38, 6, 15);
        g.rotate(-rotationBackLegUnder, x + 30 + 5 - 15, y - 36 + 5); 
        
        //reset the rotation for the hind leg
        g.rotate(-rotationBackLegUpper, x + 5, y - 30);               
        
        
        //the position of the wings
        final int wingPositionX = x - 8;
        
        //draw the shaft of the wing
        paintPart(g, shellPrimary, shellBorder, wingPositionX, y-70+flapOffset, 5, 30 - flapOffset);
        
        //draw the front of all the wing strips
        for(int i = 5; i <= 50; i+=3) {
            //calculate its bounds
            int wingX = wingPositionX + i;
            int wingY = y - 70 + 2 + flapOffset;
            int wingH = 25-i/3-flapOffset;
            
            //draw it
            paintPart(g,shellPrimary,shellBorder,wingX,wingY,3,wingH);   
        }  
        
        //draw the lower edge of the wing
        paintPart(g, shellPrimary, shellBorder,wingPositionX, y - 70 + flapOffset,50,2);
        
        //draw the back of the shaft of the wing
        paintPart(g, shellSecondary, shellBorder, wingPositionX, y - 70 + 2 + flapOffset / 2, 5, flapOffset / 2); 
        
        //draw the back of all the wing strips
        for(int i = 5; i <= 50; i+=3) {
            //calculate its bounds
            int wingX = wingPositionX+i-1;
            int wingY = y-70+2+flapOffset;
            int wingH = Math.min((25-i/3)/2+50/6,flapOffset/2);
            wingY -= wingH;
            
            //draw it
            paintPart(g, shellSecondary, shellBorder, wingX, wingY, 3, wingH);   
        }                 
        
        //reset the rotation of the dragon
        g.rotate(-rotationBody, x, y - 40 + 25 / 2); 
        
        
        //if we inverted the dragon, invert the drawing back
        if (!lookingLeft()) { 
            g.translate(x * 2,0);
            g.scale(-1,1);
        }      
        
        
        //draw the fire particles in front of the dragon
        for(FireParticle f: fires) {
            f.Draw(g,true);
        }
    }  
    

    @Override
    public EnumSet<RENDERPASS> getRenderPass() {
        return EnumSet.of(RENDERPASS.NORMAL);
    }
    


}