package dragons;

import java.util.ArrayList;

import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.Graphics2D;

import gui.IconHolder;
import gui.FireIcon;
import gui.HeartIcon;
import gui.UPDATETYPE;

import inventory.IInventory;
import inventory.Item;

import main.RENDERPASS;
import java.util.EnumSet;


public class PlayerDragon extends Dragon implements IInventory {
    
  
    private IconHolder hearts;
    private IconHolder fires;
    
    private Item[] items;
    
    public PlayerDragon(int x, int y,int health,int fire) {
        super(x, y, health, fire);
        hearts = new IconHolder(10, 10, HeartIcon.class, health);
        fires = new IconHolder(10, 60, FireIcon.class, fire);
        items = new Item[9];
        items[0] = new inventory.HealthPotion("Health Potion", 3);
        items[1] = new inventory.FirePotion("Potion of Fire", 10);
    }
    
    public void healMe(float val) {
        super.healMe(val);
        hearts.setValue(getHealth(), reviving ? UPDATETYPE.RESTORE : UPDATETYPE.DEFAULT);
    }
    
    public void damageMe(float val) {
        super.damageMe(val);
        hearts.setValue(getHealth(), UPDATETYPE.DAMAGE);
    }      
        
    public void restoreFire(float val) {
        super.restoreFire(val);
        fires.setValue(getFireMeter(), UPDATETYPE.RESTORE);
    }
    
    public void useFire(float val) {
        super.useFire(val);
        fires.setValue(getFireMeter(), UPDATETYPE.DEFAULT);
    }      
    
    
    
    
    public Color PrimaryColor() {
        return new Color(50,50,50);
    }
    public Color SecondaryColor() {
        return new Color(25,25,25);
    }
    public Color BorderColor() {
        return new Color(0,0,0);
    }
    
    public void handleKey(KeyEvent ke, boolean pressed) {
        handleKey(ke.getKeyCode(), pressed);
    }       
    
    private boolean reviving;
    
    @Override
    protected void onDeath() {
        super.onDeath();
        reviving = true;
        fire = false;
        flying = false;
        left = false;
        right = false;
    }
    
    @Override
    public void update() {
        super.update();
        if (reviving) {
            healMe(0.005F);
            if (getHealth() == getMaxHealth()) {
                reviving = false;
            }
        }
    }
    
    private void handleKey(int keycode, boolean pressed) {       
        if (!reviving && keycode == 38) {
            flying = pressed;
        }else if(keycode == 37) {
            left = pressed;
        }else if(keycode == 39) {
            right = pressed;
        }else if (!reviving && keycode == 88) {
            fire = pressed;
        }else if (keycode >= 49 && keycode <= 57){ 
            if (pressed) {
                int number = keycode - 49;
                if (items[number] != null) {
                    items[number].useItem(this);
                    if (items[number].getAmount() <= 0) {
                        items[number] = null;
                    }
                }
            }
        }else{
            //debug
            System.out.println(keycode + " : " + pressed);
        }
    }   
    
    
    @Override
    public int getInventorySize() {
        return items.length;
    }
    
    @Override
    public boolean setContent(int pos, Item item) {
        if (items[pos] == null) {
            items[pos] = item;
            return true;
        }else{
            return false;
        }
    }
    
    @Override
    public Item getContent(int pos) {
        return items[pos];
    }
    
    @Override
    public void draw(Graphics2D g, RENDERPASS pass) {
        if (pass == RENDERPASS.OVERLAY) {
            for (int i = 0; i < items.length; i++) {
                if (items[i] != null) {
                    items[i].drawItem(g, 10, 120 + i * 35, false);
                }
            }
        }else{
            super.draw(g, pass);
        }
    }
    
    @Override
    public EnumSet<RENDERPASS> getRenderPass() {
        EnumSet<RENDERPASS> ret = super.getRenderPass();
        ret.add(RENDERPASS.OVERLAY);
        return ret;
    }
    
}
