package example.items;

import java.util.List;

import net.minecraft.client.renderer.texture.IconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Icon;
import net.minecraft.world.World;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import example.blocks.BlockInfo;

public class ItemColorCharger extends Item {

	public ItemColorCharger(int id) {
		super(id);
		setCreativeTab(CreativeTabs.tabMisc);
		setHasSubtypes(true);
		setMaxStackSize(1);
		setUnlocalizedName(ItemInfo.CHARGER_UNLOCALIZED_NAME);
	}
	
	
	public int getUsesForColor(int dmg, int color) {
		return (dmg & (0b11100 << (color * 3))) >> (color * 3 + 2);
	}
	
	public int getActiveColor(int dmg) {
		return dmg & 0b11;
	}
	
	public int reduceActiveUses(int dmg) {
		int active = getActiveColor(dmg);
		int uses = getUsesForColor(dmg, active) - 1;
	
		dmg &= ~(0b11100 << (active * 3));
		dmg |= uses << (active * 3 + 2);
	
		return dmg;
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void addInformation(ItemStack stack, EntityPlayer player, List list, boolean useExtraInfo) {
		list.add("Active color: " + ItemInfo.CHARGER_COLOR_TAGS[getActiveColor(stack.getItemDamage())] + ItemInfo.CHARGER_COLORS[getActiveColor(stack.getItemDamage())]);
		for (int i = 0; i < ItemInfo.CHARGER_COLORS.length; i++) {
			int uses = getUsesForColor(stack.getItemDamage(), i);
			if (uses > 0) {
				list.add("Uses left of " + ItemInfo.CHARGER_COLOR_TAGS[i] + ItemInfo.CHARGER_COLORS[i] + ": " + uses);
			}
		}
	}
	
	@SideOnly(Side.CLIENT)
	private Icon[] icons;
	
	@Override
	@SideOnly(Side.CLIENT)
	public void registerIcons(IconRegister register) {
		icons = new Icon[ItemInfo.CHARGER_COLORS.length];
		for (int i = 0; i < icons.length; i++) {
			icons[i] = register.registerIcon(ItemInfo.TEXTURE_LOCATION + ":" + ItemInfo.CHARGER_TEXTURE + ItemInfo.CHARGER_COLORS[i].toLowerCase());
		}
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public Icon getIconFromDamage(int dmg) {
		return icons[getActiveColor(dmg)];
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubItems(int id, CreativeTabs tab, List list) {
		list.add(new ItemStack(id, 1, 0b11111111111100));
	}
	
	@Override
	public boolean onItemUseFirst(ItemStack stack, EntityPlayer player, World world, int x, int y, int z, int side, float hitX, float hitY, float hitZ) {
		if (!world.isRemote) {
			if (player.isSneaking()) {
				int dmg = stack.getItemDamage();
				int nextColor = (dmg + 1) & 0b11;
				dmg &= ~0b11;
				dmg |= nextColor;
				stack.setItemDamage(dmg);
			
				return true;
			}else if (world.getBlockId(x, y, z) == BlockInfo.COLOR_ID) {
				int meta = world.getBlockMetadata(x, y, z);
				int itemColor = getActiveColor(stack.getItemDamage());
			
				if (getUsesForColor(stack.getItemDamage(), itemColor) > 0 && (meta & 0b0011) != itemColor) {
					meta &= ~0b0011;
					meta |= itemColor;
				
					world.setBlockMetadataWithNotify(x, y, z, meta, 3);
					stack.setItemDamage(reduceActiveUses(stack.getItemDamage()));
				
					if ((stack.getItemDamage() & 0b11111111111100) == 0) {
						stack.stackSize--;
					}
				
					return true;
				}
			}
		
		}
		
		return false;
	}
}
