package gui;

import java.awt.geom.Area;
import java.awt.geom.Arc2D;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.BasicStroke;
import java.awt.GradientPaint;

import java.lang.reflect.Constructor;
import java.util.ArrayList;

import main.Game;

public abstract class Icon {
    //the position on the screen
    protected int x;
    protected int y;
    
    //the id in the Icon Holder
    private int id;
    
    //how much filed this icon is with color, 1 being completely colorized while 0 is completely empty
    private float filled;
    
    //used to move the icon when animating taken damage
    private float damageAngle;
    
    private int restoreTimer;
    private float restoreOffset;
    
    protected Icon(IconHolder owner, int id) {
        this.id = id;
        x = owner.getX() + 50 * id;
        y = owner.getY();
    }
  
    //update the value of this icon, the parameter is the total value for the Icon Holder, not the value 
    //of this specific icon, this is why we need to take the id into account to calculate the value of
    //this specific icon
    protected void updateValue(float value, UPDATETYPE type) {
        filled = Math.max(0, Math.min(1, value  - id));
        if (filled < 1 && type == UPDATETYPE.DAMAGE) {
            damageAngle = 0.1F;
        }else if(filled > 0 && filled < 1 && type == UPDATETYPE.RESTORE) {
            restoreTimer = 3;
        }
    }

    
    protected void update() {
        if (damageAngle > 0) {
            damageAngle += 0.5F;
            if (damageAngle > Math.PI * 6) {
                damageAngle = 0;
            }
            y = 10 + (int)(Math.sin(damageAngle) * 5);
        }
        if (restoreTimer > 0) {
            restoreOffset += 2.5F;
            --restoreTimer;
        }
    }


     
    protected final void draw(Graphics2D g) {
        Area area = getArea();
        
        //draw the gray background
        g.setColor(Color.gray);
        g.fill(area); 
        
        //subtrac the part that shouldn't be filled using a rectangle
        Area filledPart = new Area(area);
        filledPart.subtract(new Area(new Rectangle(x, y, 40, (int)((1-filled)*34))));
        
        //draw the red part of the background
        g.setColor(getFilledColor());
        g.fill(filledPart);  
        
        //draw the heart border
        Stroke old = g.getStroke();
        Stroke dashed = new BasicStroke (3.0F, BasicStroke.CAP_BUTT, 
                                         BasicStroke.JOIN_MITER, 
                                         10.0F, new float[] {4.0F, 2.0F, 4.0F, (getBorderLength() - 20) / 2}, restoreOffset);       
        Stroke basic = new BasicStroke(2);
        
        g.setStroke(basic);
        g.setColor(Color.black);
        g.draw(area);        

        if (restoreTimer > 0) {
            g.setStroke(dashed);
            g.setColor(getFilledColor());
            g.draw(area);  
        }

        g.setStroke(old);        
    }
    
    protected abstract Area getArea();
    protected abstract Color getFilledColor();  
    protected abstract float getBorderLength();

    
}

