package gui;

import java.util.ArrayList;
import java.util.EnumSet;
import java.lang.reflect.Constructor;
import java.awt.Graphics2D;
import main.Game;
import main.IGraphicalObject;
import main.RENDERPASS;

public class IconHolder implements IGraphicalObject{
 
    //the start position of this IconHolder
    private int x;
    private int y;
    
    //the class to use to create new icons
    private Class<? extends Icon> iconClass;
    
    //the current value and the max value
    private float value;
    private int max;
    
    //the icons
    private ArrayList<Icon> icons;
    
    public IconHolder(int x, int y, Class<? extends Icon> icon, int max) {
        this.x = x;
        this.y = y;
        iconClass = icon;
        icons = new ArrayList<Icon>();
        setMax(max);
        setValue(max, UPDATETYPE.DEFAULT);
        
        Game.instance.objects.add(this);
    }
    
    public void setMax(int max) {
        this.max = max;
        
        //create more icons if we don't have enough   
        if (icons.size() < max) {
            for (int i = icons.size(); i < max; i++) {
                createNewIcon();
            }
        }else if (icons.size() > max) {
            for (int i = icons.size() - 1; i > max; i--) {
                icons.remove(i);
            }            
        }        
    }
    
    public void setValue(float value, UPDATETYPE type) {
        this.value = value;
        for (Icon icon : icons) {
            icon.updateValue(value, type);
        }        
    }    
    
    //create a new icon with the given class, this has to be done like this since we store the class
    //in a variable rather than just have it directly in the code(so we can't do something like new IconHeart(this, 3))
    private void createNewIcon() {
        try {            
            Constructor iconConstructor = iconClass.getConstructor(new Class[] {IconHolder.class, int.class});           
            Icon icon = (Icon)iconConstructor.newInstance(new Object[] {this, icons.size()});
            icons.add(icon);
        }catch(Exception e) {
            System.out.println("Failed to create an icon of type " + iconClass);
            
            e.printStackTrace();
        }        
    }  
    
    
    @Override
    public void draw(Graphics2D g, RENDERPASS pass) {
        for (Icon icon : icons) {
            icon.draw(g);
        }
    }

    @Override
    public void update() {
        for (Icon icon : icons) {
            icon.update();
        }        
    }    
    
    protected int getX() {
        return x;
    }
    
    protected int getY() {
        return y;
    }
    
    @Override
    public EnumSet<RENDERPASS> getRenderPass() {
        return EnumSet.of(RENDERPASS.OVERLAY);
    }
    
}