package main;

import java.awt.geom.AffineTransform;
import javax.swing.JComponent;
import javax.swing.JFrame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Color;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import javax.swing.event.MouseInputAdapter;

import java.util.ArrayList;

import dragons.Dragon;
import dragons.PlayerDragon;
import dragons.EnemyDragon;

import world.Tree;
import world.Obstacle;

import gui.IconHolder;



public class Game extends JComponent {
    
    //create the window which everything will be shown in
    private JFrame frame = new JFrame("Dragons");
    
    //the dragon the player controls
    private PlayerDragon player;
    
    //the camera offset
    private int camX = 0;
    private int camY = 0;
    
    
    //used to exit the program
    private boolean exit = false;
    
    private boolean loaded;
    
   
    public static Game instance;
    public ArrayList<IGraphicalObject> objects;
    public ArrayList<Obstacle> obstacles;
    
    public static void main(String args[]){
        new Game();
    }
    
    //dispose everything and stop the game loop
    public void dispose() {
        exit = true;
        frame.setVisible(false);
        frame.dispose();
    }
    
   
    
    
    public Game() {
        super();
        
        instance = this;
        
        initializeWindow();
           

        objects = new ArrayList<IGraphicalObject>();   
        obstacles = new ArrayList<Obstacle>();   
        
        player = new PlayerDragon(250,200,5,3);
        for (int i = 0; i < 10; i++) {
            new EnemyDragon(350,200,3,5);
        }
        
        
        //create some trees
        new Tree(50,470);
        new Tree(10,470);
        new Tree(-150,470);
        new Tree(250,470);
        new Tree(550,470);
        new Tree(1050,470);
        new Tree(1100,470);
        new Tree(100,470);
        new Tree(-500,470);
        new Tree(-300,470);
        
    
        
        loaded = true;

        //the game loop
        while(!exit){            
            //update everything
            update();
            
            //repaint everything
            repaint();  
            
           
            //wait for the next frame
            try {
                Thread.sleep(50);
            }catch(Exception e) { 
                
            }
        }
        
    }
    
    
    private void initializeWindow() {
        //add the game to the window
        frame.getContentPane().add(this);
        
        //when the user clicks to exit we want to prevent the normal exit code to run...
        frame.setDefaultCloseOperation (JFrame.DO_NOTHING_ON_CLOSE);
        
        //...instead we handle it over here
        WindowListener windowListener = new WindowAdapter()
        {
            public void windowClosing ( WindowEvent w )
            {
                //call dispose to exit
                dispose();
            } 
        };
        frame.addWindowListener( windowListener );    
        
        
        //this keyadapter is used to handle key presses
        KeyAdapter ka = new KeyAdapter()
        {
            public void keyPressed(KeyEvent ke)
            {
                handleKey(ke,true);
            }  
            public void keyReleased(KeyEvent ke)
            {
                handleKey(ke,false);
            }      
        };    
        frame.getContentPane().addKeyListener(ka);
        
        //Sthis mouseinputadapter will handler mouse movements and mouse clicks
        //this is however nothing we need in this program, but you might wanna use it anyways ;)
        MouseInputAdapter ma = new MouseInputAdapter() {
            public void mouseMoved(MouseEvent  e)
            {
                
            }    
            
            public void mouseClicked(MouseEvent  e)
            {
                
            }              
        };
        frame.getContentPane().addMouseListener(ma);
        frame.getContentPane().addMouseMotionListener(ma);
        
        
        
        //pack the JFrame
        frame.pack();
        
        //show the JFrame
        frame.setVisible(true); 
        
        //set the size of the window
        frame.setSize(720+6,600);
        
        //activate the JFrame
        frame.getContentPane().setFocusable(true);
        frame.getContentPane().requestFocus();
        
        //prevents the user from resizing the window
        frame.setResizable(false);    
    }
    


    public void update() {
        
        for (IGraphicalObject o : objects) {
            o.update();
        }
        

        updateCameraPosition();
    }
    
    @Override
    public synchronized void paintComponent(Graphics gb)
    {      
        if (loaded) {          
            
            //get access to more advanced garphical methods
            Graphics2D g = (Graphics2D)gb;
            
            //activate anti-aliasing
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            
            //draw the sky
            drawSky(g);
            
            //move the camera's y-axis
            AffineTransform  oldTtransform = g.getTransform();
            g.translate(0,camY);
            
            //draw the ground
            drawGround(g);
            
            //move the camera's x-axis
            g.translate(camX,0);
            
            
            //draw the content of the world
            drawContent(g, RENDERPASS.BACKGROUND);
            drawContent(g, RENDERPASS.NORMAL);
                        

            //move the camera back
            g.setTransform(oldTtransform);
            
            drawContent(g, RENDERPASS.OVERLAY);

        }
        
    }
    
    private void drawContent(Graphics2D g, RENDERPASS pass) {
        for (IGraphicalObject o : objects) {
            if (o.getRenderPass().contains(pass)) {
                o.draw(g, pass);
            }
        }
    }
    
    //send the keyboard input to the player's dragon
    public void handleKey(KeyEvent ke, boolean pressed) {
        if (player != null) {
            player.handleKey(ke,pressed);
        }
    }
    
    //update the camera so the player will always be in the screen
    private void updateCameraPosition() {
        if (player.getX() + camX >= 600) {
            camX = 600 - (int)player.getX();
        }else if (player.getX() + camX <= 120) {
            camX = 120 - (int)player.getX();
        }
        
        if (player.getY() + camY >= 480) {
            camY = 480 - (int)player.getY();
        }else if (player.getY() + camY <= 120) {
            camY = 120 - (int)player.getY();
        }        
    }
    
    //draw the gradient sky
    private void drawSky(Graphics2D g) {
        for (int i = -30 + camY % 30; i < 600; i+= 30) {
            float bright = (i-camY+2400)/3000F;
            bright = Math.max(0,Math.min(1F,bright));
            g.setColor(Color.getHSBColor(190F/360F,1F,bright));
            g.fillRect(0,i,720,30);  
        }
    }
    
    
    private void drawGround(Graphics2D g) {
        g.setColor(new Color(200,70,0));
        g.fillRect(0,470,720,120);     
        
        g.setColor(Color.green);
        g.fillRect(0,470,720,15);         
    }
    
}