package particles;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Graphics2D;

import world.Obstacle;
import dragons.Dragon;
import main.Game;


public class FireParticle {
    //the fire particle's location
    private float x;
    private float y;
    
    //speed and acceleration
    private float speedPrimary;
    private float speedSecondary;
    private float accSecondary;
    
    //the angle at which the particle is heading
    private float angle;
    
    //used to reverse the direction when the dragon is flying hte opposite direction
    private int directionMultiplier;
    
    //the dragon's speed when the particle is created
    private int dragonSpeedX;
    private int dragonSpeedY;
    
    //used to dermine how long it takes before the particle vanishes
    private int life;
    private int startlife;
    
    //the color of the particle
    private Color c;
    
    //the obstacle this particle is stuck at, or null if it's free to fly
    private Obstacle atObstacle;
    
    //the z-position of the particle compared to the dragon, this is to allow some particles to
    //be behind the dragon while some are in the front, to make it more dynamic
    private boolean front;     
    
    //the dragon where this fire particle originates
    private Dragon owner;
    
    //skapar eldens med utgngslge i den givna positionen
    public FireParticle(Dragon owner, int x, int y, float angle, boolean invertDirection, int dragonSpeedX, int dragonSpeedY) {
        //sparar den givna positionen
        this.x = x;
        this.y = y;
        this.angle = angle;
        this.dragonSpeedX = dragonSpeedX;
        this.dragonSpeedX = dragonSpeedX;
        this.owner = owner;
        
        if (invertDirection) {
            directionMultiplier = -1;
        }else{
            directionMultiplier = 1;
        }
        
        
        //randomizes the speed of the particle
        speedPrimary = -1 - (float)(Math.random() * 2.0F);              
        speedSecondary = 0.4F - (float)(Math.random() * 0.8F);
        
        //randomize its acceleration
        accSecondary = (float)(Math.random() * 0.04F) - (float)(Math.random() * 0.04F);
        
        //randomize its z-position
        front = (int)(Math.random() * 2F) == 0 ? true : false;
        
        //randomizes the life time of this particle
        startlife = life = 10 + (int)(Math.random() * 10F) + (int)(Math.random() * 10F);
        
        //randomizes the color using the HSB(Hue-Saturation-brightness) color scale
        //this allows it to randomize in the yellow-orange-red range properly
        c = Color.getHSBColor((float)((350 + Math.random() * 70) %360)/360F,1F,1F);   
    }
    
    //used to create stationary black fire particles on objects
    private FireParticle(Dragon owner, int x, int y) {
        //saves the parameters
        this.x = x;
        this.y = y;
        this.owner = owner;  
        
        //make sure that we don't have a scorched particle here already
        for(FireParticle f : owner.getFireList()) {
            if (f.x == x && f.y == y) {
                return;
            }
        }
        
        //stores the current obstacle this particle is stuck at
        atObstacle = doesHitObstacle();
        if (atObstacle != null){
            
            //scorched particles are on obstacles and therefore always behind the dragon
            front = false;
            
            //set it to a fixed life span
            startlife = life = 105;
            
            //randomizes the color, this color will however quickly burn out to black, due to the life span
            c = Color.getHSBColor((float)((350 + Math.random() * 70) %360)/360F,1F,1F);

            owner.getFireList().add(this);
        }
        
    }
    
    
    
    
    //renders the particle
    public void Draw(Graphics2D g,boolean front) {
        //make sure that the render-pass is the correct one
        if (this.front == front) {
            g.setColor(c);
            g.fillRect((int)x, (int)y,1,1); 
        }
    }
    
    //get the obstacle that this particle is clashing with
    private Obstacle doesHitObstacle() {
        for (Obstacle o : Game.instance.obstacles) {
            
            if (o.getArea().contains(x, y)) {                
                return o;
            }
        }
        return null;
    }
    
    //updates this particle
    public boolean update() {
        //update the position of the particle
        x += speedPrimary * Math.cos(angle) * directionMultiplier + speedSecondary * Math.sin(angle) * directionMultiplier + dragonSpeedX;
        y += speedPrimary * Math.sin(angle) + speedSecondary * Math.cos(angle) +  dragonSpeedY;
        
        //accelerate
        speedSecondary += accSecondary;
        if (atObstacle == null && startlife - life >= 6) {
            atObstacle = doesHitObstacle();
            if (atObstacle != null) {
                //stop the particle
                speedPrimary = 0;
                speedSecondary = 0;
                dragonSpeedX = 0;
                dragonSpeedY = 0;
                accSecondary = 0;
                front = false;
                
                //let the ember burn for a while
                startlife = life = 200;
            }
            
        }else if(atObstacle != null && atObstacle.isFlamable()) {
            if (life < 100) {
                //when an ember is about to burn out it looks like a stationary scorched particle
                 c = Color.black;
             }else if ((int)(Math.random() * 15) == 0) {
                 //while the embers are burning they can spawn scorched particles nearby
                 int newX = (int)x;
                 int newY = (int)y;
                 
                 newX += -3 + (int)(Math.random() * 7);
                 newY += -3 + (int)(Math.random() * 7); 
                 
                 new FireParticle(owner,newX,newY);
             }
        }
        
        //returnera om elden fortfarande "lever"
        return --life > 0;               
    }
    
    
} 