package world;

import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Polygon;
import java.awt.Graphics2D;

import main.RENDERPASS;

public class Tree extends Obstacle {
    
    private Area treeArea;
    private Area leavesArea;
    private Color leavesColor;
    private Area area;
    
    //the collsion box
    @Override
    public Area getArea() {
        return area;
    }
    
    public Tree(float x, float y) {
        super(x, y);
        
        //create the stem
        Rectangle tree = new Rectangle((int)getX(),(int)getY()-100,20,100);
        treeArea = new Area();
        treeArea.add(new Area(tree));
        
        //generate the leaves
        GeneralPath leaves = new GeneralPath();
        Polygon leavesCenter = new Polygon();
        
        //randomize all the values defining a tree
        float rotation =  (float)(Math.random() * Math.PI * 2);;
        float treeParts = 8 + (int)(Math.random() * 8);
        int ellipseW = 20 + (int)(Math.random() * 20);
        int ellipseH = 20 + (int)(Math.random() * 20);        
        int leavesSize = 20 + (int)(Math.random() * 30);
        
        //loop through and generate all leaves chunks
        for(float a = rotation; a < Math.PI * 2 + rotation; a += (Math.PI * 2)/treeParts) {
            
            //generate the leave chunk
            int ellipseX  = (int)(tree.x+(int)(tree.getWidth()/2) + Math.cos(a)*leavesSize);
            int ellipseY = (int)(tree.y + Math.sin(a)*leavesSize);
            
            
            //create the center of the leaves, to make sure that there's not a space inbetween the leaveschunks           
            leavesCenter.addPoint(ellipseX,ellipseY);
            
            ellipseX -= ellipseW/2;
            ellipseY -= ellipseH/2;
            
            //add the leaves chunk to the leaves shape
            Ellipse2D.Float ellipse = new Ellipse2D.Float(ellipseX,ellipseY,ellipseW,ellipseH);            
            leaves.append(ellipse,false);
        }
        
        leavesArea = new Area(leaves);
        leavesArea.add(new Area(leavesCenter));    
        
        //randomize the color of the leaves, using the HSB(Hue-Saturatuin-Brightness) color scale
        leavesColor = Color.getHSBColor(114F/360F,1F,0.4F + (float)(Math.random() * 0.4F));
        
        //create the collision box
        area = new Area(tree);
        area.add(leavesArea);
    }
    
    @Override
    public void update() {
        
    }
    
    @Override
    public void draw(Graphics2D g, RENDERPASS pass) {
        //draw the wood
        g.setColor(new Color(100,60,20));
        g.fill(treeArea);
        
        //draw the leaves
        g.setColor(leavesColor);
        g.fill(leavesArea);
    }
    
    
    @Override
    public boolean isFlamable() {
        return true;
    }
    
}